/*****************************************
 * Large Assignment 1: L-Systems
 * Name:        Christopher Tye
 * E-mail:      ctye@unm.edu
 * Course:      CS591, Computational Fabrication
 * Submitted:   09/14/2023 
 
 Instructions:
 press 't' to set the iterating system to a new "tree" system
 press 'b' to set the iterating system to a new "bush" system
 press 'l' to set the iterating system to a new "lavender" system
 press ' ' to save a single png image next iteration
 press 's' to save png images for all consecutive iterations
 ****************************************/

import Turtle.*;
LSystemClass system;
Turtle t;
boolean constantSnapshotting;

float treeSize;
float treeAngle = 20;
float treeScale = 0.8;
String treeAxiom = "FX";
String[][] treeRule = {{"F","F"}, 
                       {"X","[F-X][F+X]","[F+X]","[F-X]","F[F-X][F+X]","[F-X][F+X]","[F-X][F+X]"},
                       {"+","+"}, 
                       {"-","-"}, 
                       {"[", "["}, 
                       {"]", "]"}};
              
float bushSize;
float bushAngle = 10;
float bushScale = 0.8;
String bushAxiom = "[-X][FX]";
String[][] bushRule = {{"F","F"}, 
                        {"X","[F--X][F+X]","F[F--X]","[F--FX][F+X]","[F--X][F+X]","[--X]"},
                        {"+","+"}, 
                        {"-","-"}, 
                        {"[", "["}, 
                        {"]", "]"}};
  
float lavenderSize;
float lavenderAngle = 20;
float lavenderScale = 0.7;
String lavenderAxiom = "[-X][FX][-X]";
String[][] lavenderRule = {{"F","FF","F","F","F"}, 
                           {"X","++","F[+X][-X]FX","F[+X][X]FX","[+X]F[X]","F[+X]FX"}, 
                           {"+","+"}, 
                           {"-","-"}, 
                           {"[", "["}, 
                           {"]", "]"}};
                             
void setup() {
  size(900, 900);
  background(#F0A5A5);
  frameRate(1);
  
  treeSize = height/2 - 100;
  bushSize = height/2 - 100;
  lavenderSize = height/2 - 100;
  
  system = new LSystemClass(lavenderAxiom, lavenderRule, lavenderSize, lavenderAngle, lavenderScale);
  t = new Turtle(this);
}

void draw() {
  iterateCurrentSystem();
}

void iterateCurrentSystem() {
  if (system != null) {
    t.setX(width/2);
    t.setY(height-10);
    background(#F0A5A5);
  
    system.drawLSystem(t);
    if (constantSnapshotting) savePicture();
    system.iterate();
  }
}

void keyPressed() {
  if (key == ' ') {
    savePicture();
  }
  if (key == 't') {
    system = new LSystemClass(treeAxiom, treeRule, treeSize, treeAngle, treeScale);
  }
  if (key == 'b') {
    system = new LSystemClass(bushAxiom, bushRule, bushSize, bushAngle, bushScale);
  }
  if (key == 'l') {
    system = new LSystemClass(lavenderAxiom, lavenderRule, lavenderSize, lavenderAngle, lavenderScale);
  }
  if (key == 's') {
    constantSnapshotting = true;
  }
}

void savePicture() {
    String filename = "LSystem" + random(0,1000000) + ".png";
    save(filename);
    println("Saved " + filename);
}
